# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoDataset_Connector.ipynb.

# %% auto 0
__all__ = ['DomoConnector']

# %% ../../nbs/classes/50_DomoDataset_Connector.ipynb 2
import httpx
import datetime as dt
from typing import List

from dataclasses import dataclass, field

import domolibrary.utils.DictDot as util_dd

import domolibrary.utils.convert as cd
import domolibrary.client.DomoAuth as dmda


# %% ../../nbs/classes/50_DomoDataset_Connector.ipynb 6
@dataclass
class DomoConnector:
    id: str
    label: str
    title: str
    sub_title: str
    description: str
    create_date: dt.datetime
    last_modified: dt.datetime
    publisher_name: str
    writeback_enabled: bool
    tags: list[str] = field(default_factory=list)
    capabilities: list[str] = field(default_factory=list)

    @classmethod
    def _from_json(cls, obj : dict):
        dd = util_dd.DictDot(obj)

        return cls(
            id=dd.databaseId,
            label=dd.label,
            title=dd.title,
            sub_title=dd.subTitle,
            description=dd.description,
            create_date=cd.convert_epoch_millisecond_to_datetime(dd.createDate),
            last_modified=cd.convert_epoch_millisecond_to_datetime(dd.lastModified),
            publisher_name=dd.publisherName,
            writeback_enabled=dd.writebackEnabled,
            tags=dd.tags,
            capabilities=dd.capabilities,
        )

# %% ../../nbs/classes/50_DomoDataset_Connector.ipynb 7
@dataclass
class DomoConnectors:
    auth :dmda.DomoAuth = field(repr = False)

    domo_connectors : List[DomoConnector] = field(default = None)
    
    async def get(
        self,
        search_text=None,
        additional_filters_ls=None,

        return_raw: bool = False,
        debug_api: bool = False,
        debug_num_stacks_to_drop=2,
        session: httpx.AsyncClient = None,
    ):
        import domolibrary.routes.datacenter as datacenter_routes

        res = await datacenter_routes.get_connectors(
            auth= self.auth,
            search_text=search_text,
            additional_filters_ls=additional_filters_ls,
            debug_api=debug_api,
            debug_num_stacks_to_drop=debug_num_stacks_to_drop,
            session=session,
            parent_class=self.__class__.__name__,
        )

        if return_raw:
            return res

        if len(res.response) == 0:
            self.domo_connectors = []

        self.domo_connectors =  [DomoConnector._from_json(obj) for obj in res.response]
        return self.domo_connectors



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoPublish.ipynb.

# %% ../../nbs/classes/50_DomoPublish.ipynb 2
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Optional, List, Any
from enum import Enum

import datetime as dt
import pandas as pd
import asyncio
import httpx

import uuid

from nbdev.showdoc import patch_to

import domolibrary.utils.DictDot as util_dd
import domolibrary.client.DomoAuth as dmda
import domolibrary.routes.publish as publish_routes

import domolibrary.classes.DomoLineage as dmdl

import domolibrary.utils.chunk_execution as dmce
import domolibrary.client.DomoError as dmde


# %% auto 0
__all__ = ['DomoPublication_Subscription', 'DomoPublication_Content_Enum', 'DomoPublication_Content',
           'DomoPublication_UnexpectedContentType', 'DomoPublication', 'DomoPublications']

# %% ../../nbs/classes/50_DomoPublish.ipynb 6
@dataclass
class DomoPublication_Subscription:
    """the subscriber represents a location a publication is sent to"""

    auth: dmda.DomoAuth = field(repr=False)
    subscription_id: str
    publication_id: str
    domain: str
    created_dt: Optional[dt.datetime] = (None,)

    parent : Any = field(repr = False, default = None)

    @classmethod
    def _from_json(cls, obj, auth: dmda.DomoAuth, parent : Any= None):

        return cls(
            auth=auth,
            subscription_id=obj.get("id") or obj.get("subscriptionId"),
            publication_id=obj["publicationId"],
            domain=obj.get("domain") or obj.get("publisherDomain"),
            created_dt=(
                (dt.datetime.fromtimestamp(obj.get("created") / 1000))
                if obj.get("created")
                else None
            ),
            parent = parent
        )

    async def get_content_details(
        self,
        debug_api: bool = False,
        debug_num_stacks_to_drop=2,
        session: httpx.AsyncClient = None,
    ):
        
        publication_content = self.parent.content

        res = await publish_routes.get_subscriber_content_details(
            auth=self.auth,
            publication_id=self.publication_id,
            subscriber_instance=self.domain,
            debug_num_stacks_to_drop=debug_num_stacks_to_drop,
            debug_api=debug_api,
            session = session,
            parent_class=self.__class__.__name__,
        )

        for content in publication_content:
            subscriber_obj = next(
                (
                    subscriber_obj
                    for subscriber_obj in res.response
                    if subscriber_obj["publisherObjectId"] == content.entity_id
                    and subscriber_obj["contentType"] == content.entity_type
                ),
                None,
            )
            if subscriber_obj is not None:
                content.subscriber_content_id = subscriber_obj["subscriberObjectId"]
                content.subscriber_insance = subscriber_obj["subscriberDomain"]

        self.content = publication_content

        return self.content

# %% ../../nbs/classes/50_DomoPublish.ipynb 9
class DomoPublication_Content_Enum(Enum):
    import domolibrary.classes.DomoDataset as dmdc
    import domolibrary.classes.DomoCard as dmac
    import domolibrary.classes.DomoPage as dmpg
    import domolibrary.classes.DomoAppStudio as dmas

    CARD = dmac.DomoCard
    DATASET = dmdc.DomoDataset
    DATA_APP = dmas.DomoAppStudio
    PAGE = dmpg.DomoPage


# %% ../../nbs/classes/50_DomoPublish.ipynb 11
@dataclass
class DomoPublication_Content:
    auth : dmda.DomoAuth

    content_id: str
    entity_type: str
    entity_id: str
    entity_domain: str
    is_v2: bool
    is_direct_content: bool

    created_dt: dt.datetime
    updated_dt: dt.datetime = None

    subscriber_content_id: str = None
    subscriber_insance: str = None

    entity: Any = field(repr=False, default=None)
    parent: Any = field(repr=False, default=None)

    """the publication content is the content from the publisher instance that is being distributed to subscribers"""

    @classmethod
    def _from_json(cls, obj: dict, auth : dmda.DomoAuth, parent: Any = None):

        entity_type = obj.get("content").get("type")
        return cls(
            auth = auth,
            content_id=obj["id"],
            entity_id=obj.get("content").get("domoObjectId"),
            entity_domain=obj.get("content").get("domain"),
            is_v2=obj.get("isV2"),
            created_dt=(
                dt.datetime.fromtimestamp(obj["created"] / 1000)
                if obj["created"]
                else None
            ),
            updated_dt=(
                dt.datetime.fromtimestamp(obj.get("content").get("updated") / 1000)
                if obj.get("content").get("updated")
                else None
            ),
            is_direct_content=obj.get("useDirectContent"),
            parent=parent,
            entity_type=entity_type,
            entity=DomoPublication_Content_Enum[entity_type].value,
        )

    async def _get_entity_lineage(self, debug_api: bool = False, session: httpx.AsyncClient = None):
        if self.entity:
            self.entity = await self.entity._get_by_id(
                auth=self.auth, id=self.entity_id, debug_api=debug_api, session=session
            )

            return hasattr(self.entity, "Lineage") and await self.entity.Lineage.get()
        
        return self.entity

    def to_api_json(self):
        return {
            "domain": self.entity_domain,
            "domoObjectId": self.entity_id,
            "customerId": self.entity_domain,
            "type": self.entity_type,
        }

# %% ../../nbs/classes/50_DomoPublish.ipynb 13
class DomoPublication_UnexpectedContentType(dmde.ClassError):
    def __init__(self, cls_instance, content_type):
        super().__init__(
            cls_instance=cls_instance,
            message=f"DomoPublication_Instantiation: Unexpected content type {content_type}",
        )


@dataclass
class DomoPublication:
    auth: dmda.DomoAuth = field(repr=False)

    id: str
    name: str
    description: str
    is_v2: bool
    created_dt: dt.datetime

    updated_dt: dt.datetime = None

    subscription_authorizations: List[DomoPublication_Subscription] = None

    content: List[DomoPublication_Content] = None

    # content_page_id_ls: List[str] = default = None
    # content_dataset_id_ls: List[str] = field(default_factory=list)
    # content_data_app_id_ls: List[str] = field(default_factory=list)

    Lineage: dmdl.DomoLineage_Publication = None

    def __post_init__(self):
        self.Lineage = dmdl.DomoLineage_Publication._for_publication(parent=self, auth=self.auth)

    def _generate_subscription_authorizations(
        self, subscription_authorizations_ls, auth
    ):
        self.subscription_authorizations = [
            DomoPublication_Subscription._from_json(obj=sub, auth=auth, parent=self)
            for sub in subscription_authorizations_ls
        ]

    def _generate_content(self, children_ls):

        self.content = [
            DomoPublication_Content._from_json(child, auth=self.auth, parent=self)
            for child in children_ls
        ]

        # if dmpc.entity_type == "PAGE":
        #     domo_pub.content_page_id_ls.append(dmpc.entity_id)

        # elif dmpc.entity_type == "DATASET":
        #     domo_pub.content_dataset_id_ls.append(dmpc.entity_id)

        # elif dmpc.entity_type == "DATA_APP":
        #     domo_pub.content_data_app_id_ls.append(dmpc.entity_id)

        # else:
        #     raise DomoPublication_UnexpectedContentType(
        #         cls_instance=domo_pub,
        #         content_type=dmpc.entity_type,
        #     )

        return self.content

    @classmethod
    def _from_json(cls, obj, auth: dmda.DomoAuth):

        domo_pub = cls(
            id=obj["id"],
            name=obj["name"],
            description=obj["description"],
            created_dt=(
                dt.datetime.fromtimestamp(obj["created"] / 1000)
                if obj["created"]
                else None
            ),
            updated_dt=(
                dt.datetime.fromtimestamp(obj.get("content").get("updated") / 1000)
                if obj.get("content").get("updated")
                else None
            ),
            is_v2=obj["isV2"],
            auth=auth,
        )

        if (
            obj.get("subscriptionAuthorizations")
            and len(obj.get("subscriptionAuthorizations")) > 0
        ):
            domo_pub._generate_subscription_authorizations(
                subscription_authorizations_ls=obj["subscriptionAuthorizations"],
                auth=auth,
            )

        if obj.get("children") and len(obj.get("children")) > 0:
            domo_pub._generate_content(obj["children"])

        return domo_pub

# %% ../../nbs/classes/50_DomoPublish.ipynb 14
@patch_to(DomoPublication, cls_method=True)
async def get_by_id(
    cls,
    publication_id,
    auth: dmda.DomoAuth = None,
    return_raw: bool = False,
    timeout=10,
    debug_api: bool = False,
    debug_num_stacks_to_drop=2,
    session: httpx.AsyncClient = None,
):
    res = await publish_routes.get_publication_by_id(
        auth=auth,
        publication_id=publication_id,
        timeout=timeout,
        debug_api=debug_api,
        debug_num_stacks_to_drop=debug_num_stacks_to_drop,
        session=session,
        parent_class=cls.__name__,
    )

    if return_raw:
        return res

    return cls._from_json(obj=res.response, auth=auth)

# | export
@patch_to(DomoPublication)
async def get_content(
    self,
    return_raw: bool = False,
    timeout=10,
    debug_api: bool = False,
    debug_num_stacks_to_drop=2,
    session: httpx.AsyncClient = None,
):
    res = await publish_routes.get_publication_by_id(
        auth=auth,
        publication_id=self.id,
        timeout=timeout,
        debug_api=debug_api,
        debug_num_stacks_to_drop=debug_num_stacks_to_drop,
        session=session,
        parent_class=self.__class__.__name__,
    )

    content = res.response.get('children')

    if return_raw:
        res.response = content
        return res
    

    if not content or len(content) == 0:
        return content
    
    return self._generate_content(content)


# %% ../../nbs/classes/50_DomoPublish.ipynb 22
@dataclass
class DomoPublications:
    auth: dmda.DomoAuth = field(repr = False)

    publications: List[DomoPublication] = field(default = None)

    subscriptions: List[DomoPublication_Subscription] = field(default = None)

    async def get(
        self,
        search_term: str = None,
        debug_api: bool = False,
        session: httpx.AsyncClient = None,
        return_raw: bool = False,
        debug_num_stacks_to_drop=2,
    ):

        res = await publish_routes.search_publications(
            auth=self.auth,
            debug_api=debug_api,
            session=session,
            search_term= search_term,
            debug_num_stacks_to_drop=debug_num_stacks_to_drop,
            parent_class=self.__class__.__name__,
        )

        if return_raw:
            return res

        self.publications = await dmce.gather_with_concurrency(
            n=60,
            *[
                DomoPublication.get_by_id(
                    publication_id=obj.get("id"), auth=self.auth
                )
                for obj in res.response
            ],
        )
        return self.publications

    async def get_subscribers(
        self,
        session: httpx.AsyncClient = None,
        return_raw: bool = False,
        debug_api: bool = False,
    ):
        """get instances subscription summaries"""

        res = await publish_routes.get_subscription_summaries(
            auth=self.auth, session=session, debug_api=debug_api
        )

        if return_raw:
            return res

        self.subscriptions = [
            DomoPublication_Subscription._from_json(sub, auth=self.auth)
            for sub in res.response
        ]

        return self.subscriptions

# %% ../../nbs/classes/50_DomoPublish.ipynb 26
@patch_to(DomoPublications)
async def search_publications(
    self: DomoPublications,
    search_term: str = None,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
    return_raw: bool = False,
    debug_num_stacks_to_drop = 2
):
    
    res = await self.get(
        search_term=search_term,
        session=session,
        debug_api=debug_api,
        return_raw = return_raw,
        debug_num_stacks_to_drop=debug_num_stacks_to_drop+1
    )

    return res

# %% ../../nbs/classes/50_DomoPublish.ipynb 28
@patch_to(DomoPublication)
async def report_content_as_dataframe(self, return_raw: bool = False):

    if return_raw:
        return await self.get_content()

    return pd.DataFrame(
        [
            {
                "plubication_id": self.id,
                "publication_name": self.name,
                "is_v2": self.is_v2,
                "publish_created_dt": self.created_dt,
                "entity_type": content.entity_type,
                "entity_id": content.entity_id,
                "entity_updated" : content.updated_dt
            }
            for content in self.content
        ]
    )


@patch_to(DomoPublication)
def report_lineage_as_dataframe(self, return_raw: bool = False):

    flat_lineage_ls = self.Lineage._flatten_lineage()

    output_ls = [
        {
            "plubication_id": self.id,
            "publication_name": self.name,
            "is_v2": self.is_v2,
            "publish_created_dt": self.created_dt,
            # "entity_type": row.get("entity_type"),
            "entity_id": row.get("entity_id"),
        }
        for row in flat_lineage_ls
    ]

    if return_raw:
        return output_ls

    return pd.DataFrame(output_ls)

# %% ../../nbs/classes/50_DomoPublish.ipynb 30
@patch_to(DomoPublication, cls_method=True)
async def create_publication(
    cls,
    name: str,
    content_ls: List[DomoPublication_Content],
    subscription_ls: List[DomoPublication_Subscription],
    unique_id: str = None,
    description: str = None,
    auth: dmda.DomoAuth = None,
    debug_api: bool = False,
):

    if not isinstance(subscription_ls, list):
        subscription_ls = [subscription_ls]

    auth = auth or cls.auth
    domain_ls = []
    content_json_ls = []
    for sub in subscription_ls:
        domain_ls.append(sub.domain)
    for content_item in content_ls:
        content_json_ls.append(content_item.to_api_json())

    if not unique_id:
        unique_id = str(uuid.uuid4())
    if not description:
        description = ""

    body = publish_routes.generate_publish_body(
        url=f"{auth.domo_instance}.domo.com",
        sub_domain_ls=domain_ls,
        content_ls=content_json_ls,
        name=name,
        unique_id=unique_id,
        description=description,
        is_new=True,
    )

    res = await publish_routes.create_publish_job(auth=auth, body=body)
    if debug_api:
        print("Create the new Publish job")
    if res.status != 200:
        print(res)
        await asyncio.sleep(2)
        res = await publish_routes.get_publication_by_id(
            auth=auth, publication_id=unique_id
        )
        if res.status != 200:
            return None
        else:
            return cls._from_json(obj=res.response, auth=auth)

    return cls._from_json(obj=res.response, auth=auth)

# %% ../../nbs/classes/50_DomoPublish.ipynb 32
@patch_to(DomoPublication)
async def update_publication(
    self,
    name: str,
    content_ls: List[DomoPublication_Content],
    subscription_ls: List[DomoPublication_Subscription],
    description: str = "",
    debug_api: bool = False,
):

    if not isinstance(subscription_ls, list):
        subscription_ls = [subscription_ls]

    domain_ls = [sub.domain for sub in subscription_ls]
    content_json_ls = [content_item.to_api_json() for content_item in content_ls]
        
    body = publish_routes.generate_publish_body(
        url=f"{self.auth.domo_instance}.domo.com",
        sub_domain_ls=domain_ls,
        content_ls=content_json_ls,
        name=name,
        unique_id=self.id,
        description=description,
        is_new=False,
    )

    res = await publish_routes.udpate_publish_job(
        auth=auth, publication_id=self.id, body=body
    )

    return res

# %% ../../nbs/classes/50_DomoPublish.ipynb 34
@patch_to(DomoPublication, cls_method=True)
async def get_subscription_invites_list(
    cls, auth: dmda.DomoAuth, debug_api: bool = False
):

    res = await publish_routes.get_subscription_invititations(
        auth=auth, debug_api=debug_api
    )
    if debug_api:
        print("Getting Publish subscription invites")

    if res.status == 200:
        return res.response
    else:
        return None


@patch_to(DomoPublication, cls_method=True)
async def accept_invite_by_id(
    cls, auth: dmda.DomoAuth, subscription_id: str, debug_api: bool = False
):

    res = await publish_routes.accept_invite_by_id(
        auth=auth, subscription_id=subscription_id, debug_api=debug_api
    )

    if res.status == 200:
        return res.response
    else:
        return None



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/ai.ipynb.

# %% auto 0
__all__ = ['generate_chat_body', 'llm_generate_text', 'OutputStyleEnum', 'generate_summarize_body', 'llm_summarize_text']

# %% ../../nbs/routes/ai.ipynb 1
import httpx
from enum import Enum

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as dmde

# %% ../../nbs/routes/ai.ipynb 4
def generate_chat_body(
    text_input: str, model="domo.domo_ai.domogpt-chat-medium-v1.1:anthropic"
):

    return {
        "input": text_input,
        "promptTemplate": {"template": "${input}"},
        "model": model,
    }

@gd.route_function
async def llm_generate_text(
    text_input,
    auth: dmda.DomoAuth,
    chat_body: dict = None,
    debug_api: bool = False,
    parent_class: str = None,
    debug_num_stacks_to_drop=1,
    return_raw: bool = False,
    session : httpx.AsyncClient = None
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/ai/v1/text/generation"

    body = chat_body or generate_chat_body(text_input=text_input)

    res = await gd.get_data(
        auth=auth,
        url=url,
        method="POST",
        body=body,
        debug_api=debug_api,
        num_stacks_to_drop = debug_num_stacks_to_drop,
        parent_class = parent_class,
        session = session
    )

    if return_raw:
        return res

    if not res.is_success:
        raise dmde.RouteError(res = res)

    res.response["output"] = res.response["choices"][0]["output"]

    return res

# %% ../../nbs/routes/ai.ipynb 7
class OutputStyleEnum(Enum):
    BULLETED = "BULLETED"
    NUMBERED = "Numbered"
    PARAGRAPH = "PARAGRAPH"


def generate_summarize_body(
    text_input: str,
    summary_length: int = 100,
    output_style: str = "BULLETED",
    system_prompt: str = "You are a helpful assistant that writes concise summaries",
    model="domo.domo_ai.domogpt-summarize-v1:anthropic",
):
    text_input = text_input if isinstance(text_input, str) else json.dumps(text_input)

    return {
        "input": text_input,
        "system": system_prompt,
        "promptTemplate": {
            "template": f"Write a {summary_length} summary of the following text. With {output_style} output.  Do not include a preamble. \nTEXT_TO_SUMMARIZE: {text_input} \nCONCISE SUMMARY:",
        },
        "model": model,
        "outputStyle": output_style,
        "outputWordLength": {"max": summary_length},
    }


@gd.route_function
async def llm_summarize_text(
    text_input: str,
    auth: dmda.DomoAuth,
    system_prompt: str = None,
    summary_length: int = 100,
    output_style: OutputStyleEnum = OutputStyleEnum.BULLETED,
    summary_body: dict = None,
    return_raw: bool = False,
    debug_api: bool = False,
    debug_num_stacks_to_drop=1,
    session: httpx.AsyncClient = None,
    parent_class: str = None 
):
    output_style = (
        output_style.value
        if isinstance(output_style, OutputStyleEnum)
        else output_style
    )

    url = f"https://{auth.domo_instance}.domo.com/api/ai/v1/text/summarize"

    body = summary_body or generate_summarize_body(
        text_input=text_input,
        system_prompt=system_prompt,
        output_style=output_style,
        summary_length=summary_length,
    )

    res = await gd.get_data(
        auth=auth,
        url=url,
        method="POST",
        body=body,
        debug_api=debug_api,
        session=session,
        num_stacks_to_drop=debug_num_stacks_to_drop,
        parent_class = parent_class,
    )

    if return_raw:
        return res

    if not res.is_success:
        raise dmde.RouteError(res = res)

    res.response["ouptput"] = res.response["choices"][0]["output"]

    return res



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoDataflow_History.ipynb.

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 2
from __future__ import annotations

from dataclasses import dataclass, field
from typing import List
import datetime as dt

import httpx

from nbdev.showdoc import patch_to

import domolibrary.utils.convert as ct
import domolibrary.utils.chunk_execution as ce

import domolibrary.client.DomoAuth as dmda
import domolibrary.routes.dataflow as dataflow_routes

# %% auto 0
__all__ = ['DomoDataflow_History_Execution', 'DomoDataflow_History']

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 3
from .DomoDataflow_Action import DomoDataflow_ActionResult

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 7
@dataclass
class DomoDataflow_History_Execution:
    auth: dmda.DomoAuth = field(repr=False)
    id: str
    dataflow_id: str
    dataflow_execution_id: str
    dataflow_version: str

    begin_time: dt.datetime
    end_time: dt.datetime
    last_updated: dt.datetime

    is_failed: bool
    state: str
    activation_type: str
    data_processor: str
    telemetry: dict
    execution_stats: dict

    action_results: List[DomoDataflow_ActionResult] = None

    @classmethod
    def _from_json(cls, de_obj, auth: dmda.DomoAuth):

        action_results = None
        if de_obj.get("actionResults"):
            action_results = [
                DomoDataflow_ActionResult._from_json(action_obj)
                for action_obj in de_obj.get("actionResults")
            ]

        return cls(
            auth=auth,
            id=de_obj["id"],
            dataflow_id=de_obj["onboardFlowId"],
            dataflow_execution_id=de_obj["dapDataFlowExecutionId"],
            dataflow_version=de_obj.get("dataFlowVersion"),
            begin_time=ct.convert_epoch_millisecond_to_datetime(
                de_obj.get("beginTime")
            ),
            end_time=ct.convert_epoch_millisecond_to_datetime(de_obj.get("endTime")),
            last_updated=ct.convert_epoch_millisecond_to_datetime(
                de_obj["lastUpdated"]
            ),
            is_failed=de_obj.get("failed"),
            state=de_obj["state"],
            activation_type=de_obj["activationType"],
            data_processor=de_obj["dataProcessor"],
            telemetry=de_obj.get("telemetry"),
            execution_stats={
                "total_bytes_written": de_obj.get("totalBytesWritten", 0),
                "total_rows_read": de_obj.get("totalRowsRead", 0),
                "total_bytes_read": de_obj.get("totalBytesRead", 0),
                "mean_download_rate_kbps": de_obj.get("meanDownloadRateKbps", 0),
                "total_rows_written": de_obj.get("totalRowsWritten", 0),
            },
            action_results=action_results,
        )

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 8
@patch_to(DomoDataflow_History_Execution, cls_method=True)
async def get_by_id(
    cls,
    auth: dmda.DomoAuth,
    dataflow_id: int,
    execution_id: int,
    debug_api: bool = False,
    debug_num_stacks_to_drop=1,
    session: httpx.AsyncClient = None,
    return_raw: bool = False,
):
    """retrieves details about a dataflow execution including actions"""

    res = await dataflow_routes.get_dataflow_execution_by_id(
        auth=auth,
        dataflow_id=dataflow_id,
        execution_id=execution_id,
        debug_api=debug_api,
        debug_num_stacks_to_drop=debug_num_stacks_to_drop,
        parent_class=cls.__name__,
        session=session,
    )

    if return_raw:
        return res

    return cls._from_json(auth=auth, de_obj=res.response)

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 12
@patch_to(DomoDataflow_History_Execution)
async def get_actions(
    self: DomoDataflow_History_Execution,
    debug_api: bool = False,
    debug_num_stacks_to_drop=1,
    session: httpx.AsyncClient = None,
    return_raw: bool = False,
):
    """retrieves details execution action results"""

    res = await dataflow_routes.get_dataflow_execution_by_id(
        auth=self.auth,
        dataflow_id=self.dataflow_id,
        execution_id=self.id,
        debug_api=debug_api,
        debug_num_stacks_to_drop=debug_num_stacks_to_drop,
        parent_class=self.__class__.__name__,
        session=session,
    )

    if return_raw:
        return res

    action_results = res.response.get("actionResults")
    if action_results:
        action_results = [
            DomoDataflow_ActionResult._from_json(action_obj)
            for action_obj in action_results
        ]

    self.action_results = action_results
    return self.action_results

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 14
@dataclass
class DomoDataflow_History:
    auth: dmda.DomoAuth = field(repr=False)
    dataflow_id: int = field(repr=False)

    dataflow: None = field(repr=False, default=None)

    execution_history: List[DomoDataflow_History_Execution] = None

# %% ../../nbs/classes/50_DomoDataflow_History.ipynb 15
@patch_to(DomoDataflow_History)
async def get_execution_history(
    self: DomoDataflow_History,
    auth: dmda.DomoAuth = None,
    maximum=10,  # maximum number of execution histories to retrieve
    debug_api: bool = False,
    debug_num_stacks_to_drop=2,
    return_raw: bool = False,
):
    """retrieves metadata about execution history.
    includes details like execution status.
    """

    auth = auth or self.auth or self.dataflow.auth

    res = await dataflow_routes.get_dataflow_execution_history(
        auth=auth,
        dataflow_id=self.dataflow_id,
        debug_api=debug_api,
        debug_num_stacks_to_drop=debug_num_stacks_to_drop,
        parent_class=self.__class__.__name__,
        maximum=maximum,
    )

    if return_raw:
        return res

    execution_history = [
        DomoDataflow_History_Execution._from_json(df_obj, auth)
        for df_obj in res.response
    ]

    await ce.gather_with_concurrency(
        *[domo_execution.get_actions() for domo_execution in execution_history], n=20
    )

    self.execution_history = execution_history

    return self.execution_history



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoDatacenter.ipynb.

# %% auto 0
__all__ = ['DomoDatacenter']

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 2
from domolibrary.routes.datacenter import (
    generate_search_datacenter_filter,
    Datacenter_Filter_Field_Enum,
    Datacenter_Filter_Field_Certification_Enum,
    Datacenter_Enum,
)

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 3
from dataclasses import dataclass
from typing import Union, List, Any
from enum import Enum

import httpx

from nbdev.showdoc import patch_to

import domolibrary.utils.chunk_execution as dmce
import domolibrary.client.DomoAuth as dmda
import domolibrary.routes.datacenter as datacenter_routes


# import domolibrary.classes.DomoDataset as dmds
# import domolibrary.classes.DomoAccount as dma
# import domolibrary.routes.card as card_routes

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 6
@dataclass
class DomoDatacenter:
    "class for quering entities in the datacenter"
    auth: dmda.DomoAuth = None

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 7
@patch_to(DomoDatacenter, cls_method=True)
async def search_datacenter(
    cls,
    auth: dmda.DomoAuth,
    maximum: int = None,  # maximum number of results to return
    body: dict = None,  # either pass a body or generate a body in the function using search_text, entity_type, and additional_filters parameters
    search_text=None,
    # can accept one value or a list of values
    entity_type: Union[str, list] = "dataset",
    additional_filters_ls=None,
    return_raw: bool = False,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> List[Any]:

    res = await datacenter_routes.search_datacenter(
        auth=auth,
        maximum=maximum,
        body=body,
        session=session,
        search_text=search_text,
        entity_type=entity_type,
        additional_filters_ls=additional_filters_ls,
        debug_api=debug_api,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 10
@patch_to(DomoDatacenter, cls_method=True)
async def search_datasets(
    cls,
    auth=dmda.DomoAuth,
    maximum: int = None,  # maximum number of results to return
    search_text=None,
    # can accept one value or a list of values
    additional_filters_ls=None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
) -> List[Any]:
    import domolibrary.classes.DomoDataset as dmds

    json_list = await cls.search_datacenter(
        auth=auth,
        maximum=maximum,
        search_text=search_text,
        entity_type=datacenter_routes.Datacenter_Enum.DATASET.value,
        additional_filters_ls=additional_filters_ls,
        return_raw=return_raw,
        session=session,
        debug_api=debug_api,
    )

    if return_raw or len(json_list) == 0:
        return json_list

    return await dmce.gather_with_concurrency(
        n=20,
        *[
            dmds.DomoDataset.get_by_id(
                dataset_id=json_obj.get("databaseId"),
                auth=auth,
                debug_api=debug_api,
                session=session,
            )
            for json_obj in json_list
        ]
    )

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 13
@patch_to(DomoDatacenter, cls_method=True)
async def get_accounts(
    cls,
    auth=dmda.DomoAuth,
    maximum: int = None,  # maximum number of results to return
    # can accept one value or a list of values
    additional_filters_ls=None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
) -> List[Any]:
    """search Domo Datacenter account api.
    Note: at the time of this writing 7/18/2023, the datacenter api does not support searching accounts by name
    """

    import domolibrary.classes.DomoAccount as dmac

    json_list = await cls.search_datacenter(
        auth=auth,
        maximum=maximum,
        entity_type=datacenter_routes.Datacenter_Enum.ACCOUNT.value,
        additional_filters_ls=additional_filters_ls,
        return_raw=return_raw,
        session=session,
        debug_api=debug_api,
    )

    if return_raw or len(json_list) == 0:
        return json_list

    domo_account_ls = [
        dmac.DomoAccount._from_json(json_obj, auth=auth) for json_obj in json_list
    ]

    return domo_account_ls

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 16
# class LineageTypes_Enum(Enum):
#     DomoDataset = "DATA_SOURCE"
#     DomoDataflow = "DATAFLOW"


# @patch_to(DomoDatacenter, cls_method=True)
# async def get_lineage_upstream(
#     cls,
#     auth: dmda.DomoAuth,
#     domo_entity,  # DomoDataset or DomoDataflow
#     return_raw: bool = False,
#     session: httpx.AsyncClient = None,
#     debug_api: bool = False,
#     debug_prn: bool = False,
# ):

#     import domolibrary.classes.DomoDataset as dmds
#     import domolibrary.classes.DomoDataflow as dmdf

#     if not session:
#         session = httpx.AsyncClient()
#         is_close_session = True

#     res = await datacenter_routes.get_lineage_upstream(
#         auth=auth,
#         entity_type=LineageTypes_Enum[domo_entity.__class__.__name__].value,
#         entity_id=domo_entity.id,
#         session=session,
#         debug_api=debug_api,
#     )

#     if return_raw or res.status != 200:
#         await session.aclose()
#         return res

#     obj = res.response

#     domo_obj = []
#     for key, item in obj.items():
#         if item.get("type") == "DATA_SOURCE":
#             domo_obj.append(
#                 await dmds.DomoDataset.get_by_id(
#                     auth=auth, dataset_id=item.get("id"), session=session
#                 )
#             )

#         if item.get("type") == "DATAFLOW":
#             # print(item.get('id'))
#             domo_obj.append(
#                 await dmdf.DomoDataflow.get_by_id(
#                     auth=auth, dataflow_id=item.get("id"), session=session
#                 )
#             )
#             pass

#     if is_close_session:
#         await session.aclose()
#     return domo_obj

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 18
@patch_to(DomoDatacenter, cls_method=True)
async def search_cards(
    cls,
    auth=dmda.DomoAuth,
    maximum: int = None,  # maximum number of results to return
    search_text=None,
    # can accept one value or a list of values
    additional_filters_ls=None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
) -> List[Any]:
    
    import domolibrary.classes.DomoCard as dmc

    json_list = await cls.search_datacenter(
        auth=auth,
        maximum=maximum,
        search_text=search_text,
        entity_type=datacenter_routes.Datacenter_Enum.CARD.value,
        additional_filters_ls=additional_filters_ls,
        return_raw=return_raw,
        session=session,
        debug_api=debug_api,
    )

    if return_raw or len(json_list) == 0:
        return json_list

    return await dmce.gather_with_concurrency(
        n=60,
        *[
            dmc.DomoCard.get_by_id(
                card_id=json_obj.get("databaseId"),
                auth=auth,
                debug_api=debug_api,
                session=session,
            )
            for json_obj in json_list
        ]
    )

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 20
@patch_to(DomoDatacenter, cls_method=True)
async def get_cards_admin_summary(
    cls,
    auth=dmda.DomoAuth,
    page_ids: List[str] = None,
    card_search_text: str = None,
    page_search_text: str = None,
    maximum: int = None,  # maximum number of results to return
    # can accept one value or a list of values
    return_raw: bool = False,
    debug_api: bool = False,
    debug_loop: bool = False,
    session: httpx.AsyncClient = None,
) -> List[Any]:
    """search Domo Datacenter card api."""

    import domolibrary.routes.card as card_routes
    import domolibrary.classes.DomoCard as dmc

    search_body = card_routes.generate_body_search_cards_admin_summary(
        page_ids=page_ids,
        card_search_text=card_search_text,
        page_search_text=page_search_text,
    )

    res = await card_routes.search_cards_admin_summary(
        auth=auth,
        body=search_body,
        maximum=maximum,
        debug_api=debug_api,
        debug_loop=debug_loop,
        session=session,
        wait_sleep=5,
    )

    if return_raw or len(res.response) == 0:
        return res

    domo_account_ls = await dmce.gather_with_concurrency(
        n=60,
        *[dmc.DomoCard._from_json(json_obj, auth=auth) for json_obj in res.response]
    )

    return domo_account_ls

# %% ../../nbs/classes/50_DomoDatacenter.ipynb 24
@patch_to(DomoDatacenter, cls_method=True)
async def search_codeengine(
    cls,
    auth=dmda.DomoAuth,
    maximum: int = None,  # maximum number of results to return
    search_text=None,
    # can accept one value or a list of values
    additional_filters_ls=None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
) -> List[Any]:

    import domolibrary.classes.DomoCodeEngine as dmceg

    search_ls = await cls.search_datacenter(
        auth=auth,
        maximum=maximum,
        search_text=search_text,
        entity_type=datacenter_routes.Datacenter_Enum.PACKAGE.value,
        additional_filters_ls=additional_filters_ls,
        return_raw=return_raw,
        session=session,
        debug_api=debug_api,
    )

    if return_raw or len(search_ls) == 0:
        return search_ls

    return await dmce.gather_with_concurrency(
        *[
            dmceg.DomoCodeEngine_Package.get_current_version_by_id(
                auth=auth,
                package_id=obj["uuid"],
            )
            for obj in search_ls
        ],
        n=10
    )
